@@ -18,7 +18,7 @@ use crate::v0;
 use crate::Location;
 
 pub(crate) use helm::Helm;
-use helper::{group_nodes_by_region, group_nodes_by_zone, is_aks, is_eks};
+use helper::{group_nodes_by_region, group_nodes_by_zone, is_aks, is_eks, NamesAndContexts};
 use kubeconfig::KubeconfigExt;
 
 mod helm;
@@ -38,54 +38,107 @@ const STATEHUB_CLUSTER_CONFIGMAP_NAME: &str = "statehub";
 #[derive(Debug)]
 pub(crate) struct Kubectl {
     kubeconfig: Kubeconfig,
+    name: v0::ClusterName,
     context: String,
 }
 
 impl Kubectl {
+    pub(crate) fn with_names_and_contexts(
+        names: Vec<v0::ClusterName>,
+        contexts: Vec<String>,
+    ) -> anyhow::Result<Vec<Self>> {
+        use NamesAndContexts::*;
+
+        let kubeconfig = Kubeconfig::read()?;
+
+        let kubectl = match (names, contexts).try_into()? {
+            GuessEverything => {
+                // Use current context for both context and name
+                let context = kubeconfig
+                    .current_context()
+                    .ok_or_else(|| anyhow::anyhow!("No current context in kubeconfig"))?
+                    .to_string();
+                let name = normalize_name(&context);
+                let kubectl = Self {
+                    kubeconfig,
+                    name,
+                    context,
+                };
+                vec![kubectl]
+            }
+            ClusterWithCurrentContext(name) => {
+                // Use current context for context and name as provided
+                let context = kubeconfig
+                    .current_context()
+                    .ok_or_else(|| anyhow::anyhow!("No current context in kubeconfig"))?
+                    .to_string();
+
+                let kubectl = Self {
+                    kubeconfig,
+                    name,
+                    context,
+                };
+                vec![kubectl]
+            }
+            NamedContext(context) => {
+                anyhow::ensure!(
+                    kubeconfig.has_context(&context),
+                    "kubeconfig doesn't have context '{}'",
+                    context
+                );
+                let name = normalize_name(&context);
+                let kubectl = Self {
+                    kubeconfig,
+                    name,
+                    context,
+                };
+                vec![kubectl]
+            }
+            ExactPairs(names_and_contexts) => names_and_contexts
+                .into_iter()
+                .map(|(name, context)| Self {
+                    kubeconfig: kubeconfig.clone(),
+                    name,
+                    context,
+                })
+                .collect(),
+        };
+
+        Ok(kubectl)
+    }
+
     pub(crate) fn new() -> anyhow::Result<Self> {
         let kubeconfig = Kubeconfig::read()?;
         let context = kubeconfig
             .current_context()
             .ok_or_else(|| anyhow::anyhow!("No current context in kubeconfig"))?
             .to_string();
+        let name = normalize_name(&context);
         Ok(Self {
             kubeconfig,
+            name,
             context,
         })
     }
 
     pub(crate) fn with_context(context: &str) -> anyhow::Result<Self> {
         let kubeconfig = Kubeconfig::read()?;
+        let name = normalize_name(context);
         let context = context.to_string();
 
         Ok(Self {
             kubeconfig,
+            name,
             context,
         })
     }
 
-    pub(crate) fn clone_for_context(&self, context: &str) -> Self {
-        let kubeconfig = self.kubeconfig.clone();
-        let context = context.to_string();
-        Self {
-            kubeconfig,
-            context,
-        }
+    pub(crate) fn _has_context(&self, name: &str) -> bool {
+        self.kubeconfig.has_context(name)
     }
 
-    pub(crate) fn has_context(&self, name: &str) -> bool {
-        self.kubeconfig
-            .contexts
-            .iter()
-            .any(|context| context.name == name)
-    }
-
-    pub(crate) fn _default_context(&self) -> Option<&str> {
-        None
-    }
-
-    pub(crate) fn cluster_name(&self) -> v0::ClusterName {
-        normalize_name(&self.context)
+    pub(crate) fn cluster_name(&self) -> &v0::ClusterName {
+        &self.name
     }
 
     async fn get_client(&self) -> anyhow::Result<Client> {
