@@ -113,10 +113,18 @@ enum Command {
         #[structopt(help = "Cluster name, defaults to current k8s context")]
         names: Vec<v0::ClusterName>,
 
+        #[structopt(
+            help = "Kubeconfig context to use for registering cluster",
+            long,
+            short,
+            visible_aliases = &["context", "ctx"]
+        )]
+        contexts: Vec<String>,
+
         #[structopt(
             help = "List of states to for this cluster to use",
             long,
-            alias = "state",
+            visible_alias = "state",
             default_value = "default"
         )]
         states: Vec<v0::StateName>,
@@ -138,7 +146,8 @@ enum Command {
         #[structopt(
             help = "The name of the state to configure as default storage class",
             long,
-            conflicts_with_all = &["no_state", "no_default_storage_class"]
+            conflicts_with_all = &["no_state", "no_default_storage_class"],
+            visible_alias = "default_sc",
         )]
         default_storage_class: Option<String>,
 
@@ -148,7 +157,8 @@ enum Command {
         #[structopt(
             help = "Namespace to install statehub components",
             long,
-            default_value = "statehub-system"
+            default_value = "statehub-system",
+            visible_alias = "ns"
         )]
         namespace: String,
 
@@ -446,6 +456,7 @@ impl Cli {
             Command::RegisterCluster {
                 non_interactive,
                 names,
+                contexts,
                 states,
                 no_state,
                 no_default_storage_class,
@@ -457,12 +468,7 @@ impl Cli {
                 cleanup_grace,
             } => {
                 let interactive = !non_interactive;
-                let kubectl = k8s::Kubectl::new()?;
-                for name in &names {
-                    if !kubectl.has_context(name) {
-                        anyhow::bail!("Context {} is not found in kubeconfig", name)
-                    }
-                }
+                let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
 
                 let no_default_storage_class = if no_state {
                     true
@@ -496,15 +502,11 @@ impl Cli {
                     // Right now interactive_register_cluster() does very much the same job as
                     // non-interactive counterpart. In the future we may handle it differently.
                     statehub
-                        .interactive_register_cluster(names, kubectl, register_cluster)
-                        .await
-                } else if names.is_empty() {
-                    statehub
-                        .register_cluster_from_current_context(kubectl, register_cluster)
+                        .interactive_register_cluster(kubectls, register_cluster)
                         .await
                 } else {
                     statehub
-                        .register_multiple_clusters(names, kubectl, register_cluster)
+                        .register_multiple_clusters(kubectls, register_cluster)
                         .await
                 }
             }
@@ -725,46 +727,21 @@ impl StateHub {
 
     async fn interactive_register_cluster(
         &self,
-        names: Vec<v0::ClusterName>,
-        kubectl: k8s::Kubectl,
+        kubectls: Vec<k8s::Kubectl>,
         register_cluster: RegisterCluster,
     ) -> anyhow::Result<()> {
-        if names.is_empty() {
-            self.register_cluster_from_current_context(kubectl, register_cluster)
-                .await
-        } else {
-            self.register_multiple_clusters(names, kubectl, register_cluster)
-                .await
-        }
-    }
-
-    async fn register_cluster_from_current_context(
-        &self,
-        kubectl: k8s::Kubectl,
-        register_cluster: RegisterCluster,
-    ) -> anyhow::Result<()> {
-        let locations = kubectl.collect_node_locations().await?;
-
-        if let Some(states) = register_cluster.states() {
-            self.adjust_all_states(states, &locations, false).await?;
-        } else {
-            self.verbosely("Skip adding this cluster to any state")?;
-        }
-
-        self.register_cluster_helper(kubectl, &locations, register_cluster)
+        self.register_multiple_clusters(kubectls, register_cluster)
             .await
     }
 
     async fn register_multiple_clusters(
         &self,
-        names: Vec<v0::ClusterName>,
-        kubectl: k8s::Kubectl,
+        kubectls: Vec<k8s::Kubectl>,
         register_cluster: RegisterCluster,
     ) -> anyhow::Result<()> {
         let mut clusters = indexmap::indexmap!();
 
-        for context in &names {
-            let kubectl = kubectl.clone_for_context(context);
+        for kubectl in kubectls {
             let cluster_locations = kubectl.collect_node_locations().await?;
             // Ideally I would store it as (kubectl => locations),
             // but Kubeconfig doesn't impl Hash, so I cannot derive it on Kubectl
